@@ -17,7 +17,7 @@ require HTML::DOM::Node;
 require HTML::DOM::NodeList::Magic;
 
 our @ISA = qw'HTML::DOM::Node';
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 
 
 {
@@ -276,6 +276,11 @@ named C<$name>, even one that is implied.
 
 This triggers a click event on the element; nothing more.
 
+=item trigger_event
+
+This overrides L<HTML::DOM::Node>'s method to trigger a DOMActivate event
+after a click.
+
 =back
 
 =cut
@@ -500,6 +505,17 @@ sub _attr_specified { defined shift->attr(shift) }
 
 sub click { shift->trigger_event('click') }
 
+sub innerHTML {
+	shift->as_HTML
+}
+
+sub starttag {
+	my $self = shift;
+	my $tag = $self->SUPER::starttag(@ _);
+	$tag =~ s/ \/>\z/>/;
+	$tag
+}
+
 # ------- OVERRIDDEN NODE METHDOS ---------- #
 
 *nodeName = \&tagName;
@@ -531,6 +547,19 @@ sub clone { # override of HTML::Element’s method; this is called
 	$clown;
 }
 
+sub trigger_event {
+	my ($a,$evnt) = (shift,shift);
+	$a->SUPER::trigger_event(
+		$evnt,
+		click_default =>sub {
+			$_[0]->target->trigger_event(DOMActivate =>
+				detail => eval{$_[0]->detail}
+			);;
+		},
+		@_,
+	);
+}
+
 
 =head1 SEE ALSO
 
@@ -549,21 +578,21 @@ L<HTML::DOM/CLASSES AND DOM INTERFACES>
 # ------- HTMLHtmlElement interface ---------- #
 
 package HTML::DOM::Element::HTML;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub version { shift->attr('version' => @_) }
 
 # ------- HTMLHeadElement interface ---------- #
 
 package HTML::DOM::Element::Head;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub profile { shift->attr('profile' => @_) }
 
 # ------- HTMLLinkElement interface ---------- #
 
 package HTML::DOM::Element::Link;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub disabled {
 	if(@_ > 1) {
@@ -622,7 +651,7 @@ sub removeAttributeNode {
 # ------- HTMLTitleElement interface ---------- #
 
 package HTML::DOM::Element::Title;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 # This is what I call FWP (no lexical vars):
 sub text {
@@ -637,7 +666,7 @@ sub text {
 # ------- HTMLMetaElement interface ---------- #
 
 package HTML::DOM::Element::Meta;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub content   { shift->attr('content'    => @_) }
 sub httpEquiv { shift->attr('http-equiv' => @_) }
@@ -647,7 +676,7 @@ sub scheme    { shift->attr('scheme'     => @_) }
 # ------- HTMLBaseElement interface ---------- #
 
 package HTML::DOM::Element::Base;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *href =\& HTML::DOM::Element::Link::href;
 *target =\& HTML::DOM::Element::Link::target;
@@ -655,7 +684,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLIsIndexElement interface ---------- #
 
 package HTML::DOM::Element::IsIndex;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub form     { (shift->look_up(_tag => 'form'))[0] || () }
 sub prompt   { shift->attr('prompt'  => @_) }
@@ -663,7 +692,7 @@ sub prompt   { shift->attr('prompt'  => @_) }
 # ------- HTMLStyleElement interface ---------- #
 
 package HTML::DOM::Element::Style;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *disabled = \&HTML::DOM::Element::Link::disabled;
 *media =\& HTML::DOM::Element::Link::media;
@@ -683,7 +712,7 @@ sub sheet {
 # ------- HTMLBodyElement interface ---------- #
 
 package HTML::DOM::Element::Body;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub aLink      { shift->attr( aLink      => @_) }
 sub background { shift->attr( background => @_) }
@@ -701,7 +730,7 @@ sub vLink      { shift->attr('vlink'     => @_) }
 # ------- HTMLUListElement interface ---------- #
 
 package HTML::DOM::Element::UL;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub compact { shift->attr( compact => @_) }
 sub type { lc shift->attr( type => @_) }
@@ -709,7 +738,7 @@ sub type { lc shift->attr( type => @_) }
 # ------- HTMLOListElement interface ---------- #
 
 package HTML::DOM::Element::OL;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub start { shift->attr( start => @_) }
 *compact=\&HTML::DOM::Element::UL::compact;
@@ -718,28 +747,28 @@ sub start { shift->attr( start => @_) }
 # ------- HTMLDListElement interface ---------- #
 
 package HTML::DOM::Element::DL;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLDirectoryElement interface ---------- #
 
 package HTML::DOM::Element::Dir;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLMenuElement interface ---------- #
 
 package HTML::DOM::Element::Menu;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLLIElement interface ---------- #
 
 package HTML::DOM::Element::LI;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *type =\& HTML::DOM::Element::Link::type;
 sub value { shift->attr( value => @_) }
@@ -747,49 +776,49 @@ sub value { shift->attr( value => @_) }
 # ------- HTMLDivElement interface ---------- #
 
 package HTML::DOM::Element::Div;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub align { lc shift->attr( align => @_) }
 
 # ------- HTMLParagraphElement interface ---------- #
 
 package HTML::DOM::Element::P;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLHeadingElement interface ---------- #
 
 package HTML::DOM::Element::Heading;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLQuoteElement interface ---------- #
 
 package HTML::DOM::Element::Quote;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub cite { shift->attr( cite => @_) }
 
 # ------- HTMLPreElement interface ---------- #
 
 package HTML::DOM::Element::Pre;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub width { shift->attr( width => @_) }
 
 # ------- HTMLBRElement interface ---------- #
 
 package HTML::DOM::Element::Br;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub clear { lc shift->attr( clear => @_) }
 
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::BaseFont;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub color { shift->attr( color => @_) }
 sub face  { shift->attr( face  => @_) }
@@ -798,7 +827,7 @@ sub size  { shift->attr( size  => @_) }
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::Font;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *color =\& HTML::DOM::Element::BaseFont::color;
 *face =\& HTML::DOM::Element::BaseFont::face;
@@ -807,7 +836,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLHRElement interface ---------- #
 
 package HTML::DOM::Element::HR;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 sub noShade  { shift->attr( noshade  => @_) }
@@ -817,7 +846,7 @@ sub noShade  { shift->attr( noshade  => @_) }
 # ------- HTMLModElement interface ---------- #
 
 package HTML::DOM::Element::Mod;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *cite =\& HTML::DOM::Element::Quote::cite;
 sub dateTime  { shift->attr( datetime  => @_) }
@@ -825,7 +854,7 @@ sub dateTime  { shift->attr( datetime  => @_) }
 # ------- HTMLAnchorElement interface ---------- #
 
 package HTML::DOM::Element::A;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub accessKey  { shift->attr(               accesskey  => @_) }
 *   charset    =\&HTML::DOM::Element::Link::charset           ;
@@ -843,21 +872,22 @@ sub shape      { shift->attr(               shape      => @_) }
 sub blur  { shift->trigger_event('blur') }
 sub focus { shift->trigger_event('focus') }
 
-# ~~~ This code is repeated almost verbatim twice in Element/Form.pm. Is there any better way to write it?
 sub trigger_event {
 	my ($a,$evnt) = (shift,shift);
 	$a->SUPER::trigger_event(
 		$evnt,
-		(ref $evnt && eval{$evnt->type} || $evnt) =~ /^click\z/i
-		 && $a->ownerDocument->default_event_handler_for('link')
-		 || @_
+		DOMActivate_default => 
+			$a->ownerDocument->
+				default_event_handler_for('link')
+		,
+		@_,
 	);
 }
 
 # ------- HTMLImageElement interface ---------- #
 
 package HTML::DOM::Element::Img;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub lowSrc  { shift->attr(               lowsrc  => @_) }
 *   name  = \&HTML::DOM::Element::Meta::name            ;
@@ -876,7 +906,7 @@ sub vspace   { shift->attr(              vspace   => @_) }
 # ------- HTMLObjectElement interface ---------- #
 
 package HTML::DOM::Element::Object;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *form=\&HTML::DOM::Element::IsIndex::form;
 sub code  { shift->attr(               code  => @_) }
@@ -901,7 +931,7 @@ sub contentDocument{}
 # ------- HTMLParamElement interface ---------- #
 
 package HTML::DOM::Element::Param;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *name=\&HTML::DOM::Element::Meta::name;
 *type=\&HTML::DOM::Element::Link::type;
@@ -911,7 +941,7 @@ sub valueType{lc shift->attr(valuetype=>@_)}
 # ------- HTMLAppletElement interface ---------- #
 
 package HTML::DOM::Element::Applet;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 * align    = \ & HTML::DOM::Element::Div::align       ;
 * alt      = \ & HTML::DOM::Element::Img::alt         ;
@@ -928,7 +958,7 @@ sub object { shift -> attr ( object => @_ ) }
 # ------- HTMLMapElement interface ---------- #
 
 package HTML::DOM::Element::Map;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub areas { # ~~~ I need to make this cache the resulting collection obj
 	my $self = shift;
@@ -949,7 +979,7 @@ sub areas { # ~~~ I need to make this cache the resulting collection obj
 # ------- HTMLAreaElement interface ---------- #
 
 package HTML::DOM::Element::Area;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 * accessKey = \ & HTML::DOM::Element::A::accessKey     ;
 * alt       = \ & HTML::DOM::Element::Img::alt         ;
@@ -963,7 +993,7 @@ sub noHref { shift -> attr ( nohref => @_ ) }
 # ------- HTMLScriptElement interface ---------- #
 
 package HTML::DOM::Element::Script;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 * text    = \ &HTML::DOM::Element::Title::text   ;
 sub htmlFor { shift -> attr ( for   => @_ )      }
@@ -976,7 +1006,7 @@ sub defer   { shift -> attr ( defer => @_ )      }
 # ------- HTMLFrameSetElement interface ---------- #
 
 package HTML::DOM::Element::FrameSet;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub rows { shift -> attr ( rows   => @_ )      }
 sub cols   { shift -> attr ( cols => @_ )      }
@@ -984,7 +1014,7 @@ sub cols   { shift -> attr ( cols => @_ )      }
 # ------- HTMLFrameElement interface ---------- #
 
 package HTML::DOM::Element::Frame;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 sub frameBorder { lc shift -> attr ( frameBorder  => @_ )      }
 sub longDesc    { shift -> attr ( longdesc     => @_ )      }
@@ -999,7 +1029,7 @@ sub contentDocument{ shift->{_HTML_DOM_doc} ||= new HTML::DOM }
 # ------- HTMLIFrameElement interface ---------- #
 
 package HTML::DOM::Element::IFrame;
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 our @ISA = 'HTML::DOM::Element';
 *align  = \&HTML::DOM::Element::Div::align;
 *frameBorder = \&HTML::DOM::Element::Frame::frameBorder;
